# -*- coding: utf-8 -*-

#import the module numpy
#np is the alias used in the following
import numpy as np

#create a matrix on the fly
#this is a conversion of a list structure
a = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])
print(a)

#type of the new structure
print(type(a))

#type of the data
print(a.dtype)

#number of dimensions
print(a.ndim)

#number of rows and columns
#shape is a tuple !
print(a.shape)

#total number of values (rows x columns)
print(a.size)

#creation and implicit typing
a = np.array([[1,2],[4,7]])
print(a.dtype)

#creation and explicit typing
a = np.array([[1,2],[4,7]],dtype=float)
print(a.dtype)

#creation from a sequence of numbers
#the dimension must be compatible
a = np.arange(0,10).reshape(2,5)
print(a)

#a vector can be transformed into a matrix
a = np.array([2.1,3.4,6.7,8.1,3.5,7.2])
print(a.shape)

b = a.reshape(3,2)
print(b.shape)
print(b)

#repeating 8 times the value 0
a = np.zeros(shape=(2,4))
print(a)

#more generally
a = np.full(shape=(2,4),fill_value=0.1)
print(a)

#modify the default directory
import os
os.chdir("... set your directory here ...")

#load the values from a data file
a = np.loadtxt("matrice.txt",delimiter="\t",dtype=float)
print(a)

#"list" of values
lst = [1.2,3.1,4.5,6.3]
print(type(lst))

#converting from a list
a = np.asarray(lst,dtype=float).reshape(2,2)
print(a)

#
#redimensioning
#

#matrx
a = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])

#add a new row
#can be used for the concatenation of 2 matrices
b = np.array([[4.1,2.6]])
c = np.append(a,b,axis=0)
print(c)

#add a new column
d = np.array([[7.8],[6.1],[5.4]])
e = np.append(a,d,axis=1)
print(e)

#insertion
f = np.insert(a,1,b,axis=0)
print(f)

#deleting
g = np.delete(a,1,axis=0)
print(g)

#modify the dimension of an existing matrix
#reading the values row by row
h = np.resize(a,new_shape=(2,3))
print(h)

#####
#EXTRACTIONS
#####

v = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])

#all the values
print(v)
#or
print(v[:,:])

#indexed access indicé - first value
print(v[0,0])

#last value
print(v[v.shape[0]-1,v.shape[1]-1])

#contiguous indices
print(v[0:2,:])

#extreme - first to 2 (not included)
print(v[:2,:])

#extreme - 1 to last
print(v[1:,:])

#negative index
print(v[-1,:])

#negative indices
print(v[-2:,:])

#using a vector of indices
#non continguous indices
m = np.array([0,2],dtype=int)
print(v[m,:])

#indexing with a vector of booleans
#if b is too short, the remainder is considered False
b = np.array([True,False,True],dtype=bool)
print(v[b,:])

#e.g. extract the row for which the sum is the lowest

#calculate the sum of columns for each row
s = np.sum(v,axis=1)
print(s)

#b is a vector of booleans
b = (s == np.min(s))
print(b)
print(v[b,:])

#sorting and searching

#get the max of rows for each columns
print(np.max(v,axis=0))

#get the max of columns for each row
print(np.max(v,axis=1))

#get the index of the max within rows for each column
print(np.argmax(v,axis=0))

#sort the rows for each column
print(np.sort(v,axis=0))

#get the sorted indices of rows for each column
print(np.argsort(v,axis=0))

#ITERATOR
v = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])

#indexed nested loop
s = 0.0
for i in range(0,v.shape[0]):
    for j in range(0,v.shape[1]):
        print(v[i,j])
        s = s + v[i,j]
print("Somme = ",s)

#iterator - row by row
s = 0.0
for x in np.nditer(v):
    print(x)
    s = s + x
print("Somme = ",s)
    
#iterator - column by column
#"F" for Fortran order
s = 0.0
for x in np.nditer(v,order="F"):
    print(x)
    s = s +x
print("Somme = ",s)
    
#see -- http://docs.scipy.org/doc/numpy/reference/arrays.nditer.html

#statistical routines
v = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])

#mean of rows for each column
print(np.mean(v,axis=0))

#mean of columns for each row
print(np.mean(v,axis=1))

#cumulative sum of values in row for each column
print(np.cumsum(v,axis=0))

#correlation matrix
#rowvar = 0 - the variables are organized in columns
m = np.corrcoef(v,rowvar=0)
print(m)

#################
#matrix calculations
#################
x = np.array([[1.2,2.5],[3.2,1.8],[1.1,4.3]])
y = np.array([[2.1,0.8],[1.3,2.5]])

#transposition
print(np.transpose(x))

#multiplication
print(np.dot(x,y))

#determinant
print(np.linalg.det(y))

#inverse
print(np.linalg.inv(y))

#solve a linear matrix equation
z = np.array([1.7,1.0])
print(np.linalg.solve(y,z))

#checking
print(np.dot(np.linalg.inv(y),z))

#eigenvalues and eigenvectors
s = np.dot(np.transpose(x),x)
print(s)

print(np.linalg.eigh(s))
